import { ActionFormData } from "@minecraft/server-ui";

/*
const buttonExample = {
  translation_key: "sg_desired_paths.guide.main.button.0",
  icon_path: "textures/items/iron_ingot", //can be undefined, button will have no icon then.
  action: showPage("main", 0, []), //can be any function or undefined. If undefined, the button will close the menu.
};
*/
function log(...args) {
  world
    .getDimension("overworld")
    .runCommandAsync(
      `tellraw @a[tag=sg_desired_paths.Debug] {"rawtext":[{"text":"${args
        .map((d) => process(d))
        .join(" ")
        .replaceAll('"', '\\"')}"}]}`
    )
    .then()
    .catch();
}

export const showGuidePage = (player, pageName, entries, buttons) => {
  if (!buttons || !buttons.length) {
    log(`Failed to show Page[${pageName}]: Requires Button.`);
    return;
  }
  const body = { rawtext: [] };
  for (let i = 0; i < entries; i++) {
    body.rawtext.push({
      translate: `sg_desired_paths.guide.${pageName}.${i}`,
      with: ["\n"],
    });
    body.rawtext.push({ text: "§r\n" });
  }
  const form = new ActionFormData();
  form.title({ rawtext: [{ translate: `sg_desired_paths.guide.${pageName}.title` }] });
  form.body(body);
  for (const button of buttons) {
    form.button(
      { rawtext: [{ translate: button.translation_key }] },
      button.icon_path
    );
  }
  form.show(player).then((response) => {
    if (response.canceled || response.selection === undefined) return;
    const action = buttons[response.selection].action;
    if (!action) return;
    action(player);
  });
};

export const showPage = (player, title, body, buttons) => {
  if (!buttons || !buttons.length) {
    log(`Failed to show Page: Requires Button.`);
    return;
  }
  const form = new ActionFormData();
  form.title(title);
  form.body(body);
  for (const button of buttons) {
    form.button(
      { rawtext: [{ translate: button.translation_key }] },
      button.icon_path
    );
  }
  form.show(player).then((response) => {
    if (response.canceled || response.selection === undefined) return;
    const action = buttons[response.selection].action;
    if (!action) return;
    action(player);
  });
};

export class RawtextBuilder {
  constructor() {
    this.body = [];
  }

  push(...elements) {
    for (const element of elements) {
      this.body.push(element);
    }
  }

  pushLine(...elements) {
    this.push(...elements);
    this.push({ text: "§r\n" });
  }

  build() {
    return { rawtext: this.body };
  }
}